const mysql = require('mysql')
const { customLogBack } = require('iperlink_error_logger')
const { runParallel } = require('iperlink_parallel_funct')


const MYSQL_SERVER = process.env.host


const makeConnection = params => {
  const host = (params.host) ? params.host : MYSQL_SERVER
  const connectionParams = {
    ...params,
    connectionLimit: 50,
    host,
    multipleStatements: params.multiple,
    queueLimit: 20,
  }
  return mysql.createConnection(connectionParams)
}


const makePool = params => {
  const connectionParams = {
    ...params,
    connectionLimit: 50,
    multipleStatements: params.multiple || false,
  }
  return mysql.createPool(connectionParams)
}

const connect = params => {
  const connectProomise = new Promise(
    (resolve, reject) => {
      const connection = makeConnection(params)
      connection.connect(err => {
        if (err) {
          reject(err)
        }
        resolve(connection)
      })
    },
  )
  return connectProomise
}

const makeAndConnect = async params => {
  try {
    const conn = await connect(params)
    return conn
  } catch (err) {
    const errLog = customLogBack({
      code: err.code,
      err: 'cant connect to db',
      params,
    }, true)
    return errLog
  }
}

const makeMultipleConnections = params => {
  runParallel(params, makeAndConnect)
}

const makeSingleMysqlConnection = async params => {
  try {
    const connection = await makeAndConnect(params)
    return connection
  } catch (err) {
    return err
  }
}

const makeMysqlPoolCluster = params => {
  const poolCluster = mysql.createPoolCluster()
  params.map(param => {
    poolCluster.add(param.name, param)
    return null
  })
  return poolCluster
}

const getMysqlClusterConn = async (cluster, name, type = '') => {
  const Proomise = new Promise(
    (resolve, reject) => {
      const connName = `${name}_${type}`
      cluster.getConnection(connName, (err, connection) => {
        if (err) {
          reject(err)
        } resolve(connection)
      })
    })
  return Proomise
}

const execQuery = async (conn, query, params = []) => {
  const Proomise = new Promise(
    (resolve, reject) => {
      conn.query(query, params, (err, results) => {
        if (err) {
          reject(err)
        } resolve(results)
      })
    })
  return Proomise
}

const getClientDb = (client, dbList) => {
  if (!dbList[client]) {
    return customLogBack({
      err: `Client ${client} does not exist`,
    }, true, false)
  }
  const { dbName, hostName } = dbList[client]

  return { dbName, hostName }
}

const prepareQuery = async (queryString, client, dbList, cluster, parsedValues, type, testing) => {
  const {
    dbName,
    hostName,
  } = getClientDb(client, dbList)
  const conn = await getMysqlClusterConn(cluster, hostName, type)
  const queryParts = queryString.split('|')
  const query = ` ${queryParts[0]}${dbName}${queryParts[1]}`
  if (testing) {
    return { formated: mysql.format(query, parsedValues) }
  }
  const result = await execQuery(conn, query, parsedValues)
  conn.release()

  return result
}

const parallelQueryExec = (params, cb) => {
  const {
    conn,
    queryString,
  } = params
  conn.query(queryString, params, (err, results) => {
    if (err) {
      cb(err)
    } cb(null, results)
  })
}

const parallelQuery = (queryStrings, conn, cb) => {
  const params = _.map(queryStrings, queryString => ({ conn, queryString }))
  runParallel(params, parallelQueryExec, cb)
}
exports.getClientDb = getClientDb
exports.parallelQuery = parallelQuery
exports.makeMysqlPoolCluster = makeMysqlPoolCluster
exports.makePool = makePool
exports.getMysqlClusterConn = getMysqlClusterConn
exports.makeAndConnect = makeAndConnect
exports.makeMultipleConnections = makeMultipleConnections
exports.makeSingleMysqlConnection = makeSingleMysqlConnection
exports.execQuery = execQuery
exports.prepareQuery = prepareQuery
